@@ -41,8 +41,6 @@
 #include "tgtadm.h"
 #include "crc32c.h"
 
-#define MAX_QUEUE_CMD	128
-
 int default_nop_interval;
 int default_nop_count;
 
@@ -348,7 +346,7 @@ static void text_scan_login(struct iscsi_connection *conn)
 				text_key_add_reject(conn, key);
 				continue;
 			}
-			if (idx == ISCSI_PARAM_MAX_XMIT_DLENGTH)
+			if (idx >= ISCSI_PARAM_FIRST_LOCAL)
 				conn->session_param[idx].val = val;
 			else
 				param_set_val(session_keys,
@@ -357,7 +355,7 @@ static void text_scan_login(struct iscsi_connection *conn)
 
 			switch (conn->session_param[idx].state) {
 			case KEY_STATE_START:
-				if (idx == ISCSI_PARAM_MAX_XMIT_DLENGTH)
+				if (idx >= ISCSI_PARAM_FIRST_LOCAL)
 					break;
 				memset(buf, 0, sizeof(buf));
 				param_val_to_str(session_keys, idx, val, buf);
@@ -405,7 +403,7 @@ static int text_check_param(struct iscsi_connection *conn)
 	for (i = 0, cnt = 0; session_keys[i].name; i++) {
 		if (p[i].state == KEY_STATE_START && p[i].val != session_keys[i].def) {
 			if (conn->state == STATE_LOGIN) {
-				if (i == ISCSI_PARAM_MAX_XMIT_DLENGTH) {
+				if (i >= ISCSI_PARAM_FIRST_LOCAL) {
 					if (p[i].val > session_keys[i].def)
 						p[i].val = session_keys[i].def;
 					p[i].state = KEY_STATE_DONE;
@@ -1092,7 +1090,8 @@ static int iscsi_cmd_rsp_build(struct iscsi_task *task)
 	rsp->cmd_status = scsi_get_result(&task->scmd);
 	rsp->statsn = cpu_to_be32(conn->stat_sn++);
 	rsp->exp_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn);
-	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn + MAX_QUEUE_CMD);
+	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn +
+				     conn->session->max_queue_cmd);
 
 	iscsi_rsp_set_residual(rsp, &task->scmd);
 
@@ -1152,7 +1151,8 @@ static int iscsi_data_rsp_build(struct iscsi_task *task)
 		datalen = maxdatalen;
 
 	rsp->exp_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn);
-	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn + MAX_QUEUE_CMD);
+	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn +
+				     conn->session->max_queue_cmd);
 
 	conn->rsp.datasize = datalen;
 	hton24(rsp->dlength, datalen);
@@ -1227,6 +1227,14 @@ void iscsi_free_task(struct iscsi_task *task)
 	conn->tp->free_data_buf(conn, scsi_get_in_buffer(&task->scmd));
 	conn->tp->free_data_buf(conn, scsi_get_out_buffer(&task->scmd));
 
+	/*
+	 * If freeing task before in/out buffers are set, make sure to free
+	 * task->data or it leaks.
+	 */
+	if ((task->data != scsi_get_in_buffer(&task->scmd)) &&
+	    (task->data != scsi_get_out_buffer(&task->scmd)))
+		conn->tp->free_data_buf(conn, task->data);
+
 	conn->tp->free_task(task);
 	conn_put(conn);
 }
@@ -1831,7 +1839,8 @@ static int iscsi_logout_tx_start(struct iscsi_task *task)
 	rsp->itt = task->req.itt;
 	rsp->statsn = cpu_to_be32(conn->stat_sn++);
 	rsp->exp_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn);
-	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn + MAX_QUEUE_CMD);
+	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn +
+				     conn->session->max_queue_cmd);
 
 	return 0;
 }
@@ -1848,7 +1857,8 @@ static int iscsi_noop_in_tx_start(struct iscsi_task *task)
 	rsp->ttt = task->req.ttt;
 	rsp->statsn = cpu_to_be32(conn->stat_sn);
 	rsp->exp_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn);
-	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn + MAX_QUEUE_CMD);
+	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn +
+				     conn->session->max_queue_cmd);
 
 	/* TODO: honor max_burst */
 	conn->rsp.datasize = task->len;
@@ -1880,7 +1890,8 @@ static int iscsi_noop_out_tx_start(struct iscsi_task *task, int *is_rsp)
 		rsp->ttt = cpu_to_be32(ISCSI_RESERVED_TAG);
 		rsp->statsn = cpu_to_be32(conn->stat_sn++);
 		rsp->exp_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn);
-		rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn + MAX_QUEUE_CMD);
+		rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn +
+					     conn->session->max_queue_cmd);
 
 		/* TODO: honor max_burst */
 		conn->rsp.datasize = task->len;
@@ -1904,7 +1915,8 @@ static int iscsi_tm_tx_start(struct iscsi_task *task)
 
 	rsp->statsn = cpu_to_be32(conn->stat_sn++);
 	rsp->exp_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn);
-	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn + MAX_QUEUE_CMD);
+	rsp->max_cmdsn = cpu_to_be32(conn->session->exp_cmd_sn +
+				     conn->session->max_queue_cmd);
 
 	return 0;
 }
