@@ -97,8 +97,6 @@ static int mmc_rw(int host_no, struct scsi_cmd *cmd)
 		break;
 	}
 
-	cmd->scsi_cmd_done = target_cmd_io_done;
-
 	offset = scsi_rw_offset(cmd->scb);
 	cmd->offset = (offset << MMC_BLK_SHIFT);
 
@@ -190,9 +188,15 @@ static int mmc_read_toc(int host_no, struct scsi_cmd *cmd)
 
 	switch (toc_format) {
 	case 0:	/* formatted toc */
-		if (toc_track != 1) {
+		if (toc_track != 0 && toc_track != 1) {
 			/* we only do single session data disks so only
-			   the first track (track 1) is valid */
+			   the first track (track 1) exists.
+			   Since this command returns the data for all 
+			   tracks equal to toc_track or higher,
+			   we must allow either track 0 or track 1
+			   as valid and both will return the data for
+			   track 1.
+			*/
 			scsi_set_in_resid_by_actual(cmd, 0);
 			sense_data_build(cmd, NOT_READY,
 					 ASC_INVALID_FIELD_IN_CDB);
@@ -2189,14 +2193,14 @@ static int mmc_mode_sense(int host_no, struct scsi_cmd *cmd)
 	return spc_mode_sense(host_no, cmd);
 }
 
-static int mmc_lu_init(struct scsi_lu *lu)
+static tgtadm_err mmc_lu_init(struct scsi_lu *lu)
 {
 	struct backingstore_template *bst;
 	struct mmc_info *mmc;
 
 	mmc = zalloc(sizeof(struct mmc_info));
 	if (!mmc)
-		return -ENOMEM;
+		return TGTADM_NOMEM;
 
 	lu->xxc_p = mmc;
 
@@ -2239,7 +2243,9 @@ static int mmc_lu_init(struct scsi_lu *lu)
 	/* Caching Page */
 	add_mode_page(lu, "8:0:10:0:0:0:0:0:0:0:0:0:0");
 	/* Control page */
-	add_mode_page(lu, "10:0:10:2:0:0:0:0:0:0:0:2:0");
+	add_mode_page(lu, "0x0a:0:10:2:0:0:0:0:0:0:0:2:0");
+	/* Control Extensions mode page:  TCMOS:1 */
+	add_mode_page(lu, "0x0a:1:0x1c:0x04:0x00:0x00");
 	/* Power Condition */
 	add_mode_page(lu, "0x1a:0:10:8:0:0:0:0:0:0:0:0:0");
 	/* Informational Exceptions Control page */
@@ -2261,10 +2267,10 @@ static int mmc_lu_init(struct scsi_lu *lu)
 			  "0x10:0:0x96:0:0:0:0:0:0:0:0:0:0:0:0:0:0:"
 			  "0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:"
 			  "0:0");
-	return 0;
+	return TGTADM_SUCCESS;
 }
 
-static int mmc_lu_online(struct scsi_lu *lu)
+static tgtadm_err mmc_lu_online(struct scsi_lu *lu)
 {
 	struct mmc_info *mmc = dtype_priv(lu);
 	struct stat st;
@@ -2287,13 +2293,7 @@ static int mmc_lu_online(struct scsi_lu *lu)
 			mmc->current_profile = PROFILE_DVD_ROM;
 	}
 
-	return 0;
-}
-
-static int mmc_lu_offline(struct scsi_lu *lu)
-{
-	lu->attrs.online = 0;
-	return 0;
+	return TGTADM_SUCCESS;
 }
 
 static struct device_type_template mmc_template = {
@@ -2301,7 +2301,7 @@ static struct device_type_template mmc_template = {
 	.lu_init	= mmc_lu_init,
 	.lu_config	= spc_lu_config,
 	.lu_online	= mmc_lu_online,
-	.lu_offline	= mmc_lu_offline,
+	.lu_offline	= spc_lu_offline,
 	.lu_exit	= spc_lu_exit,
 	.ops		= {
 		{spc_test_unit,},
@@ -2338,7 +2338,7 @@ static struct device_type_template mmc_template = {
 		{spc_start_stop,},
 		{spc_illegal_op,},
 		{spc_illegal_op,},
-		{spc_start_stop,}, /* allow medium removal */
+		{spc_prevent_allow_media_removal,},
 		{spc_illegal_op,},
 
 		/* 0x20 */
