@@ -12,9 +12,10 @@ enum data_direction {
 };
 
 struct scsi_data_buffer {
-	int resid;
-	uint32_t length;
 	uint64_t buffer;
+	uint32_t length;
+	uint32_t transfer_len;
+	int32_t resid;
 };
 
 struct scsi_cmd {
@@ -34,6 +35,7 @@ struct scsi_cmd {
 
 	uint64_t cmd_itn_id;
 	uint64_t offset;
+	uint32_t tl;
 	uint8_t *scb;
 	int scb_len;
 	uint8_t lun[8];
@@ -45,8 +47,6 @@ struct scsi_cmd {
 	unsigned char sense_buffer[SCSI_SENSE_BUFFERSIZE];
 	int sense_len;
 
-	void (*scsi_cmd_done)(struct scsi_cmd *, int);
-
 	struct list_head bs_list;
 
 	struct it_nexus *it_nexus;
@@ -81,27 +81,48 @@ static inline type scsi_get_##dir##_##field(struct scsi_cmd *scmd)		\
 	return get_cast (scmd->dir##_sdb.field);				\
 }
 
-scsi_data_buffer_accessor(length, uint32_t,,);
-scsi_data_buffer_accessor(resid, int,,);
+scsi_data_buffer_accessor(length, uint32_t, ,);
+scsi_data_buffer_accessor(transfer_len, uint32_t, ,);
+scsi_data_buffer_accessor(resid, int32_t, ,);
 scsi_data_buffer_accessor(buffer, void *, (unsigned long), (void *)(unsigned long));
 
 static inline void scsi_set_in_resid_by_actual(struct scsi_cmd *scmd,
-					       uint32_t actual)
+					       uint32_t transfer_len)
 {
-	scsi_set_in_resid(scmd, scsi_get_in_length(scmd) - actual);
+	uint32_t expected_len = scsi_get_in_length(scmd);
+	int32_t resid;
+
+	if (transfer_len <= expected_len)
+		resid = expected_len - transfer_len;
+	else {
+		resid = -(int32_t)(transfer_len - expected_len);
+		transfer_len = expected_len;
+	}
+	scsi_set_in_transfer_len(scmd, transfer_len);
+	scsi_set_in_resid(scmd, resid);
 }
 
 static inline void scsi_set_out_resid_by_actual(struct scsi_cmd *scmd,
-					       uint32_t actual)
+						uint32_t transfer_len)
 {
-	scsi_set_out_resid(scmd, scsi_get_out_length(scmd) - actual);
-}
+	uint32_t expected_len = scsi_get_out_length(scmd);
+	int32_t resid;
 
+	if (transfer_len <= expected_len)
+		resid = expected_len - transfer_len;
+	else {
+		resid = -(int32_t)(transfer_len - expected_len);
+		transfer_len = expected_len;
+	}
+	scsi_set_out_transfer_len(scmd, transfer_len);
+	scsi_set_out_resid(scmd, resid);
+}
 
 enum {
 	TGT_CMD_QUEUED,
 	TGT_CMD_PROCESSED,
 	TGT_CMD_ASYNC,
+	TGT_CMD_NOT_LAST,
 };
 
 #define CMD_FNS(bit, name)						\
@@ -121,3 +142,4 @@ static inline int cmd_##name(const struct scsi_cmd *c)			\
 CMD_FNS(QUEUED, queued)
 CMD_FNS(PROCESSED, processed)
 CMD_FNS(ASYNC, async)
+CMD_FNS(NOT_LAST, not_last)
