@@ -735,10 +735,13 @@ static int report_opcodes_all(struct scsi_cmd *cmd, int rctd,
 	data = &buf[4];
 
 	ops = cmd->dev->dev_type_template.ops;
-	for (i = 0; i < 256; i++) {
+	for (i = 0; i < NR_SCSI_OPCODES; i++) {
 		if (ops[i].cmd_perform == spc_illegal_op)
 			continue;
 
+		if (!is_bs_support_opcode(cmd->dev->bst, i))
+			continue;
+
 		/* this command does not take a service action, so just
 		   report the opcode
 		*/
@@ -826,6 +829,9 @@ static int report_opcode_one(struct scsi_cmd *cmd, int rctd, uint8_t opcode,
 
 	ops = cmd->dev->dev_type_template.ops;
 
+	if (!is_bs_support_opcode(cmd->dev->bst, opcode))
+		return SAM_STAT_CHECK_CONDITION;
+
 	if ((is_service_action && !ops[opcode].service_actions)
 	||  (!is_service_action && ops[opcode].service_actions)) {
 		return SAM_STAT_CHECK_CONDITION;
@@ -1869,7 +1875,7 @@ enum {
 	Opt_optimal_xfer_gran, Opt_optimal_xfer_len,
 	Opt_removable, Opt_readonly, Opt_online,
 	Opt_mode_page,
-	Opt_path,
+	Opt_path, Opt_bsopts,
 	Opt_bsoflags, Opt_thinprovisioning,
 	Opt_err,
 };
@@ -1890,6 +1896,7 @@ static match_table_t tokens = {
 	{Opt_online, "online=%s"},
 	{Opt_mode_page, "mode_page=%s"},
 	{Opt_path, "path=%s"},
+	{Opt_bsopts, "bsopts=%s"},
 	{Opt_bsoflags, "bsoflags=%s"},
 	{Opt_thinprovisioning, "thin_provisioning=%s"},
 	{Opt_err, NULL},
@@ -1921,7 +1928,7 @@ tgtadm_err lu_config(struct scsi_lu *lu, char *params, match_fn_t *fn)
 	attrs = &lu->attrs;
 	lu_vpd = attrs->lu_vpd;
 
-	if (!strncmp("targetOps", params, 9))
+	if (params && !strncmp("targetOps", params, 9))
 		params = params + 10;
 
 	while ((p = strsep(&params, ",")) != NULL) {
